<?php
namespace Ak\Orm\Service;

use Catequil\Core\Mapping\HasInputFilter;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Repository\RepositoryFactory as RepositoryFactoryInterface;
use Doctrine\Persistence\ObjectRepository;
use Laminas\InputFilter\InputFilterAwareInterface;
use Laminas\InputFilter\InputFilterPluginManager;
use ReflectionClass;
use ReflectionException;

/**
 * Class RepositoryFactory
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class RepositoryFactory implements RepositoryFactoryInterface
{
    private RepositoryPluginManager $plugins;
    private InputFilterPluginManager $filters;
    private array $repos = [];

    /**
     * @param RepositoryPluginManager $plugins
     * @param InputFilterPluginManager $filters
     */
    public function __construct(
        RepositoryPluginManager $plugins,
        InputFilterPluginManager $filters
    ) {
        $this->plugins = $plugins;
        $this->filters = $filters;
    }

    /**
     * Gets the repository for an entity class.
     *
     * @param EntityManagerInterface $entityManager The EntityManager instance.
     * @param string $entityName The name of the entity.
     *
     * @return ObjectRepository
     *
     * @throws ReflectionException
     */
    public function getRepository(EntityManagerInterface $entityManager, $entityName): ObjectRepository
    {
        $hash = $entityManager->getClassMetadata($entityName)->getName() . spl_object_hash($entityManager);

        if (isset($this->repos[$hash])) {
            return $this->repos[$hash];
        }

        $repo = $this->plugins->get($entityName);

        if ($repo instanceof InputFilterAwareInterface) {
            $reflClass = new ReflectionClass($entityName);
            $reflAttribute = $reflClass->getAttributes(HasInputFilter::class)[0] ?? null;

            if ($reflAttribute) {
                $attribute = $reflAttribute->newInstance();
                $params = $attribute->getArguments();
                $filterName = $params['inputFilter'];
                $filter = $this->filters->get($filterName);
                $repo->setInputFilter($filter);
            }
        }

        return $this->repos[$hash] = $this->plugins->get($entityName);
    }
}
